# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneclient.auth.identity import v2 as auth_v2
from keystoneclient import discover as keystone_discover
from keystoneclient import session
from keystoneclient.v2_0 import client as keystone_v2
from keystoneclient.v3 import client as keystone_v3


def create_keystone_client(**kwargs):
    discover = keystone_discover.Discover(**kwargs)
    for version_data in discover.version_data():
        version = version_data["version"]
        if version[0] <= 2:
            return keystone_v2.Client(**kwargs)
        elif version[0] == 3:
            return keystone_v3.Client(**kwargs)
    raise Exception(
        'Failed to discover keystone version for url %(auth_url)s.', **kwargs)


def create_keystone_session(cacert, **kwargs):
    auth = auth_v2.Password(**kwargs)
    return session.Session(auth=auth, verify=cacert)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import time

from shaker.openstack.clients import glance
from shaker.openstack.clients import heat
from shaker.openstack.clients import keystone
from shaker.openstack.clients import neutron
from shaker.openstack.clients import nova


# As of now only Nova and Neutron clients support Keystone sessions.
# Thus the only way to create clients is from keystone client instance
# and auth token. The token gets expired in an hour and there is no
# way to automatically refresh it. So the current implementation is to
# recreate keystone client from scratch

MODERN_CLIENT_MAKERS = {
    'neutron': neutron.create_client,
    'nova': nova.create_client,
}
OLD_CLIENT_MAKERS = {
    'glance': glance.create_client,
    'heat': heat.create_client,
}

KEYSTONE_AUTH_EXPIRATION = 60


class OpenStackClientProxy(object):
    def __init__(self, keystone_creator, client_creator):
        self.keystone_creator = keystone_creator
        self.client_creator = client_creator
        self.last_update_time = 0

    def __getattribute__(self, name):
        if name in ['keystone_creator', 'client_creator',
                    'client', 'last_update_time']:
            return super(OpenStackClientProxy, self).__getattribute__(name)
        else:
            now = int(time.time())
            if now > self.last_update_time + KEYSTONE_AUTH_EXPIRATION:
                self.last_update_time = now
                self.client = self.client_creator(
                    keystone_client=self.keystone_creator())
            return self.client.__getattribute__(name)


class OpenStackClient(object):
    def __init__(self, username, password, tenant_name, auth_url, region_name,
                 cacert):
        self.region_name = region_name or 'RegionOne'
        self.cacert = cacert or ''
        self._osc_cache = {}
        self.keystone_creator = functools.partial(
            keystone.create_keystone_client,
            username=username, password=password,
            tenant_name=tenant_name, auth_url=auth_url, cacert=cacert)
        self.session_creator = functools.partial(
            keystone.create_keystone_session, cacert,
            username=username, password=password,
            tenant_name=tenant_name, auth_url=auth_url)
        # ping OpenStack
        self.keystone_creator()

    def __getattribute__(self, name):
        if name != '_osc_cache' and name in self._osc_cache:
            return self._osc_cache[name]

        client = None
        if name in MODERN_CLIENT_MAKERS:
            session = self.session_creator()
            client = MODERN_CLIENT_MAKERS[name](session, self.region_name)
        elif name in OLD_CLIENT_MAKERS:
            client_creator = functools.partial(
                OLD_CLIENT_MAKERS[name], os_region_name=self.region_name,
                cacert=self.cacert)
            client = OpenStackClientProxy(self.keystone_creator,
                                          client_creator)

        if client:
            self._osc_cache[name] = client
            return client
        else:
            return super(OpenStackClient, self).__getattribute__(name)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from shaker.engine.executors import netperf


IP = '10.0.0.10'
AGENT = {'slave': {'ip': IP}}


class TestNetperfWrapperExecutor(testtools.TestCase):

    def test_get_command(self):
        executor = netperf.NetperfWrapperExecutor({}, AGENT)

        expected = {'data': ('netperf-wrapper -H %s -l 60 -s 1 '
                             '-f csv tcp_download') % IP,
                    'type': 'program'}
        self.assertEqual(expected, executor.get_command())

    def test_get_command_with_params(self):
        executor = netperf.NetperfWrapperExecutor(
            dict(method='ping', time=10, interval=0.5), AGENT)

        expected = {'data': ('netperf-wrapper -H %s -l 10 -s 0.5 '
                             '-f csv ping') % IP,
                    'type': 'program'}
        self.assertEqual(expected, executor.get_command())

    def test_process_reply(self):
        executor = netperf.NetperfWrapperExecutor({}, AGENT)
        message = {
            'stdout': """tcp_download,Ping ICMP,TCP download
0.0,0.09,
2.0,0.0800211283506,
4.0,0.0602545096056,
6.0,0.0502416561724,28555.9
8.0,0.05,25341.9871721
10.0,0.0500947171761,30486.4518264
12.0,0.0603484557656,
14.0,0.0603987445198,
"""
        }
        expected = {
            'samples': [
                [0.0, 0.09, None],
                [2.0, 0.0800211283506, None],
                [4.0, 0.0602545096056, None],
                [6.0, 0.0502416561724, 28555.9],
                [8.0, 0.05, 25341.9871721],
                [10.0, 0.0500947171761, 30486.4518264],
                [12.0, 0.0603484557656, None],
                [14.0, 0.0603987445198, None],
            ],
            'meta': [
                ['time', 's'], ['Ping ICMP', 'ms'], ['TCP download', 'Mbit/s'],
            ]
        }
        reply = executor.process_reply(message)
        self.assertEqual(expected['samples'], reply['samples'],
                         message='Samples data')
        self.assertEqual(expected['meta'], reply['meta'],
                         message='Metadata')
